﻿#Requires -Version 3.0

<#
.SYNOPSIS
Creates and deploys a Microsoft Azure Website, azure redis cache and azure storage
For more detailed documentation go to: http://go.microsoft.com/fwlink/?LinkID=394471 
PS: Set the script folder as currentfolder before running this script
.EXAMPLE
PS C:\scriptfolder> .\Publish-WebApplicationWebSite.ps1 -EnvConfiguration .\Configurations\MyAzureWebsiteSample-WAWS-dev.json -mydomain "<your domainname>"

#>

[CmdletBinding(HelpUri = 'http://go.microsoft.com/fwlink/?LinkID=391696')]
param
(
    [Parameter(Mandatory = $false)]
    [ValidateScript({Test-Path $_ -PathType Leaf})]
    [String]
    $EnvConfiguration = ".\Configurations\MyAzureWebsiteSample-WAWS-dev.json",

    [Parameter(Mandatory = $false)]
    [ValidateScript({Test-Path $_ -PathType Leaf})]
    [String]
    $WebDeployPackage = ".\MyAzureWebsiteSample.zip",

    [Parameter(Mandatory = $true)]
    [String]
    $mydomain
)



function Read-ConfigFile
{
    [CmdletBinding()]
    param
    (
        [Parameter(Mandatory = $true)]
        [ValidateScript({Test-Path $_ -PathType Leaf})]
        [String]
        $ConfigurationFile
    )

    Write-Verbose 'Read-ConfigFile: Start'

    # Get the contents of the JSON file (-raw ignores line breaks) and convert it to a PSCustomObject
    $config = Get-Content $ConfigurationFile -Raw | ConvertFrom-Json

    if (!$config)
    {
        throw ('Read-ConfigFile: ConvertFrom-Json failed: ' + $error[0])
    }

  

    # Build a hash table from the values in the PSCustomObject
    $returnObject = New-Object -TypeName Hashtable

    $returnObject.Add('env', $config.environmentSettings.webSite.env)
    $returnObject.Add('name', $config.environmentSettings.webSite.name)
    $returnObject.Add('subscriptionId', $config.environmentSettings.webSite.subscriptionId)
    $returnObject.Add('location', $config.environmentSettings.webSite.location)
    $returnObject.Add('certificateFileName', $config.environmentSettings.webSite.certificateFileName)
    $returnObject.Add('clientId', $config.environmentSettings.webSite.clientId)
    $returnObject.Add('appInsightsKey', $config.environmentSettings.webSite.appInsightsKey)
    $returnObject.Add('redisExpiryHours', $config.environmentSettings.webSite.redisExpiryHours)

    Write-Verbose 'Read-ConfigFile: End'

    return $returnObject
}


# Script main routine


try
{
    Start-Transcript -Path ".\MyAzureWebsiteSample.log"
    
    
    # The script has been tested on Powershell 3.0
    Set-StrictMode -Version 3

    # Check if Windows Azure Powershell is avaiable

    if ((Get-Module -ListAvailable Azure) -eq $null)
    {
        throw "Windows Azure Powershell not found! Please install from http://www.windowsazure.com/en-us/downloads/#cmd-line-tools  ,  http://go.microsoft.com/?linkid=9811175&clcid=0x409"
    }
    # Check if Windows Azure Powershell is avaiable

    if ((Get-Module -ListAvailable MSOnline) -eq $null)
    {
        throw "Windows Azure AD Powershell not found! Please install from http://msdn.microsoft.com/en-us/library/azure/jj151815.aspx"
    }

    Switch-AzureMode -Name AzureServiceManagement

    # Following modifies the Write-Verbose behavior to turn the messages on globally for this session
    #$VerbosePreference = "Continue"
    $ErrorActionPreference = "Stop"

    $EnvConfig = Read-ConfigFile $EnvConfiguration 

    if (!(Test-Path -Path $EnvConfig.certificateFileName))
    {
        throw "Certificate file not found. Please copy certificate file to the current folder and run script again. File: " + $EnvConfig.certificateFileName;
    }


    #Derive Names for Env
    $Env = $EnvConfig.env.ToLower();
    $MyResourceGroupName = "Myresourcegroup$Env".ToLower();
    $siteName = $EnvConfig.name.ToString();
    $MyWebSiteName = "$siteName$Env".ToLower();
    $MyRedisCacheName = "Myrediscache$Env".ToLower();
    $MyLogStorageAccountName = "MyWebsiteLogStorage$Env".ToLower();
    $MyLogStorageTableName = $MyWebSiteName + "logs";
    $Location = $EnvConfig.location;
    $AppIdUri = "https://microsoft.onmicrosoft.com/$MyWebSiteName";
    $PackagePath = $WebDeployPackage;




    #Set current azure account
    $UserCred = Get-Credential -Message "Enter your Domain credentials"
    if ($UserCred -eq $null)
    {
        throw "Need domain credentials to proceed..Exiting.."
    }

    Add-AzureAccount -Credential $UserCred
    Select-AzureSubscription -SubscriptionId $EnvConfig.subscriptionId -Current 

    #Upload Certificte to Azure aad application
    Connect-Msolservice -Credential $UserCred
    $cer = New-Object System.Security.Cryptography.X509Certificates.X509Certificate
    $certName = $EnvConfig.certificateFileName.ToLower();
    $cer.Import("$pwd\$certName")
    $binCert = $cer.GetRawCertData()
    $credValue = [System.Convert]::ToBase64String($binCert);
    $creds = Get-MsolServicePrincipalCredential -AppPrincipalId $EnvConfig.clientId -ReturnKeyValues 1

    $credExists = 0;

    foreach ($cred in $creds) {
        if ($cred.Value -eq $credValue) {
            $credExists = 1;
            break;
        }
    }

    if (!$credExists) {
        New-MsolServicePrincipalCredential -AppPrincipalId $EnvConfig.clientId -Type asymmetric -Value $credValue -StartDate $cer.GetEffectiveDateString() -EndDate $cer.GetExpirationDateString() -Usage verify
    }
    Get-MsolServicePrincipalCredential -AppPrincipalId $EnvConfig.clientId  -ReturnKeyValues 1



    
    #Now start creating resources
    Switch-AzureMode -Name AzureResourceManager

    Write-Verbose "Check if resourcegroup $MyResourceGroupName exists.." 

    $MyResourceGroup = Get-AzureResourceGroup | Where-Object {$_.ResourceGroupName -eq $MyResourceGroupName}
    if ($MyResourceGroup -eq $null) 
    {   
        Write-Verbose "Not found, Creating Resource Group '$MyResourceGroupName'.." 
        $MyResourceGroup = New-AzureResourceGroup -Name $MyResourceGroupName -Location $Location
        Write-Verbose "Created Resource Group '$MyResourceGroupName'!" 
    }
    else
    {
        Write-Verbose "Resource Group '$MyResourceGroupName' already exists..Skipping creation." 
    }

    #Am also adding Azure Redis cache to my app
    Write-Verbose "Check if Redis Cache $MyResourceGroupName exists.." 

    $MyRedisCache = Get-AzureRedisCache | Where-Object {$_.ResourceGroupName -eq $MyResourceGroupName -and $_.Name -eq $MyRedisCacheName}
    if ($MyRedisCache -eq $null) 
    {   
        Write-Verbose "Not found, Creating Redis Cache '$MyRedisCacheName'." 
        #Microsoft.Cache/Redis
        $MyRedisCache = New-AzureRedisCache -Name $MyRedisCacheName -ResourceGroupName $MyResourceGroupName -Location $Location
    }
    else
    {
        Write-Verbose "Redis Cache '$MyRedisCacheName' already exists..Skipping creation." 
    }
    $MyRedisCacheKeys = Get-AzureRedisCacheKey -Name $MyRedisCacheName -ResourceGroupName $MyResourceGroupName
    $MyRedisCacheKey = $MyRedisCacheKeys.PrimaryKey

    # Create the website 
    $resourceType = "Microsoft.Web/sites"
    Write-Verbose "Check if Website $MyWebSiteName exists.." 
    $website = Get-AzureResource | Where-Object {$_.Name -eq $MyWebSiteName -and $_.ResourceGroupName -eq $MyResourceGroupName -and $_.ResourceType -eq $resourceType}
    if ($website -eq $null) 
    {   
        Write-Verbose "Not found, Creating website '$MyWebSiteName'.." 

        $website = New-AzureResource -Name $MyWebSiteName -ResourceType $resourceType -ResourceGroupName $MyResourceGroupName -Location $Location -ApiVersion "2014-04-01" -PropertyObject @{}
        Write-Verbose "Created website '$MyWebSiteName'." 
    }
    else
    {
        Write-Verbose "Website '$MyWebSiteName' already exists..Skipping creation." 
    }



    Switch-AzureMode -Name AzureServiceManagement

    # Create log storage account if it does not already exist.

    Write-Verbose "Check if Storage account $MyLogStorageAccountName exists.." 
    $logStorageAccount = Get-AzureStorageAccount | Where-Object { $_.StorageAccountName -eq $MyLogStorageAccountName }
    if($logStorageAccount -eq $null) 
    {
        Write-Verbose "Not found, Creating storage account '$MyLogStorageAccountName'."
        $logStorageAccount = New-AzureStorageAccount -StorageAccountName $MyLogStorageAccountName -Location $Location
    }
    else
    {
        Write-Verbose "Storage account '$MyLogStorageAccountName' already exists..Skipping creation."
    }



    #Create log storage table if it does not exist

    $storageAccountKey = Get-AzureStorageKey -StorageAccountName $MyLogStorageAccountName
    $context = New-AzureStorageContext -StorageAccountName $MyLogStorageAccountName -StorageAccountKey $storageAccountKey.Primary
    $logTable = Get-AzureStorageTable -Context $context | Where-Object { $_.Name -eq $MyLogStorageTableName}

    if($logTable -eq $null) 
    {
        Write-Verbose "Creating log table '$MyLogStorageTableName'."
        $logTable = New-AzureStorageTable -Name $MyLogStorageTableName -Context $context
    }
    else
    {
        Write-Verbose "Storage table '$MyLogStorageTableName' already exists..Skipping creation."
    }



    #Override AppSettings
    $appSettings = @{
                        "CurrentEnv"   = $EnvConfig.env;
                        "ida:Audience" = $AppIdUri;
                        "ida:Tenant"   = "$mydomain.onmicrosoft.com";
                        "ida:RedisConnectionString"   = "$MyRedisCacheName.redis.cache.windows.net,ssl=true,password=$MyRedisCacheKey";
                        "ida:RedisExpiryTimeoutHours"   = $EnvConfig.redisExpiryHours;
                        "AppInsightsKey"   = $EnvConfig.appInsightsKey;
                    } 

    # Link the website to the storage account
    Write-Verbose "Linking storage account '$MyLogStorageAccountName' to '$MyWebSiteName'."

    # Setting appsettings to website '$MyWebSiteName'
    Set-AzureWebsite -Name $MyWebSiteName -AppSettings $appSettings -DetailedErrorLoggingEnabled 1

    #Enabling diagnostics on the website '$MyWebSiteName'

    Write-Verbose "Enabling diagnostices.."
    Enable-AzureWebsiteApplicationDiagnostic -Name $MyWebSiteName -Verbose -LogLevel Information -TableStorage -StorageAccountName $MyLogStorageAccountName -StorageTableName $MyLogStorageTableName
    Write-Verbose "Diagnostices enabled..."


    #Now publish the website
    Write-Verbose "Publishing '$MyWebSiteName'."
    Publish-AzureWebsiteProject -Package $PackagePath -Name $MyWebSiteName

    Write-Verbose "Restarting '$MyWebSiteName'."
    Restart-AzureWebsite -Name $MyWebSiteName

    Write-Verbose "My Azure Website Sample deployed successfully to $Env environment!"

    
}
finally
{
   Stop-Transcript
    Write-Verbose "Stopping Transcript. "
}

